{---------------------------------------------------------------------------
 EF_DBEDT - Labeled data-aware input controls for Borland's Delphi product.
 v. 1.07a February 6, 1997 (for Delphi 1 and 2)
 Affordable Shareware; costs only U.S. $5 (together with EF_EDIT)
----------------------------------------------------------------------------
 Copyright  1996 Eric G.V. Fookes
 - All Rights Reserved -
 Address: Av. Eugne-Pittard 22Ter, CH - 1206 Geneva, Switzerland
 E-mail: fookes@sc2a.unige.ch
 Web: http://www.unige.ch/sciences/terre/geologie/fookes/delphi.htm
----------------------------------------------------------------------------}
unit Ef_DbEdt;

interface

uses
  WinTypes, WinProcs, Classes, Graphics, Messages, Controls, StdCtrls, SysUtils,
  DBCtrls, Ef_edit;

type
  TDbLblEdit = class(TDbEdit)
    protected
     FLabel:TLabel;
     FLabelPos:TLabelPos;
     FMargin:Integer;
     FAddColon,FRepositionLabel:Boolean;
     procedure Loaded; override;
     procedure VisibleChanging; override;
     procedure WMEnable(var Msg:TMessage); message WM_ENABLE;
     procedure SetBounds(ALeft,ATop,AWidth,AHeight:Integer); override;
     procedure DestroyLabel;
     procedure InitLabel;
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;
     procedure OnLabelNewBounds(Sender: TObject);
     procedure DrawLabel;
     procedure SetLabelCaption(const AString:string);
     function  GetLabelCaption:string;
     procedure SetLabelPos(APos:TLabelPos);
     procedure SetMargin(AMargin:Integer);
     procedure SetAddColon(SetOn:Boolean);
     procedure SetLabel(ALabel:TLabel);
    public
     constructor Create(AOwner: TComponent); override;
     destructor Destroy; override;
     function HasLabel:Boolean;
    published
     property LabelText:string read GetLabelCaption write SetLabelCaption stored False;
     property LabelPos:TLabelPos read FLabelPos write SetLabelPos default DefaultLabelPos;
     property LabelMargin:Integer read FMargin write SetMargin default DefaultMargin;
     property LabelColon:Boolean read FAddColon write SetAddColon default DefaultAddColon;
     property LabelComponent:TLabel read FLabel write SetLabel;
  end;

  TDbLblMemo = class(TDbMemo)
    protected
     FLabel:TLabel;
     FLabelPos:TLabelPos;
     FMargin:Integer;
     FAddColon,FRepositionLabel:Boolean;
     procedure Loaded; override;
     procedure VisibleChanging; override;
     procedure WMEnable(var Msg:TMessage); message WM_ENABLE;
     procedure SetBounds(ALeft,ATop,AWidth,AHeight:Integer); override;
     procedure DestroyLabel;
     procedure InitLabel;
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;
     procedure OnLabelNewBounds(Sender: TObject);
     procedure DrawLabel;
     procedure SetLabelCaption(const AString:string);
     function  GetLabelCaption:string;
     procedure SetLabelPos(APos:TLabelPos);
     procedure SetMargin(AMargin:Integer);
     procedure SetAddColon(SetOn:Boolean);
     procedure SetLabel(ALabel:TLabel);
    public
     constructor Create(AOwner: TComponent); override;
     destructor Destroy; override;
     function HasLabel:Boolean;
    published
     property LabelText:string read GetLabelCaption write SetLabelCaption stored False;
     property LabelPos:TLabelPos read FLabelPos write SetLabelPos default DefaultLabelPos;
     property LabelMargin:Integer read FMargin write SetMargin default DefaultMargin;
     property LabelColon:Boolean read FAddColon write SetAddColon default DefaultAddColon;
     property LabelComponent:TLabel read FLabel write SetLabel;
  end;

  TDbLblComboBox = class(TDbComboBox)
    protected
     FLabel:TLabel;
     FLabelPos:TLabelPos;
     FMargin:Integer;
     FAddColon,FRepositionLabel:Boolean;
     procedure Loaded; override;
     procedure VisibleChanging; override;
     procedure WMEnable(var Msg:TMessage); message WM_ENABLE;
     procedure SetBounds(ALeft,ATop,AWidth,AHeight:Integer); override;
     procedure DestroyLabel;
     procedure InitLabel;
     procedure Notification(AComponent: TComponent; Operation: TOperation); override;
     procedure OnLabelNewBounds(Sender: TObject);
     procedure DrawLabel;
     procedure SetLabelCaption(const AString:string);
     function  GetLabelCaption:string;
     procedure SetLabelPos(APos:TLabelPos);
     procedure SetMargin(AMargin:Integer);
     procedure SetAddColon(SetOn:Boolean);
     procedure SetLabel(ALabel:TLabel);
    public
     constructor Create(AOwner: TComponent); override;
     destructor Destroy; override;
     function HasLabel:Boolean;
    published
     property LabelText:string read GetLabelCaption write SetLabelCaption stored False;
     property LabelPos:TLabelPos read FLabelPos write SetLabelPos default DefaultLabelPos;
     property LabelMargin:Integer read FMargin write SetMargin default DefaultMargin;
     property LabelColon:Boolean read FAddColon write SetAddColon default DefaultAddColon;
     property LabelComponent:TLabel read FLabel write SetLabel;
  end;

procedure Register;

implementation

constructor TDbLblEdit.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   FRepositionLabel := True;
   FLabelPos := DefaultLabelPos;
   FMargin := DefaultMargin;
   FAddColon := DefaultAddColon;
end;

destructor TDbLblEdit.Destroy;
begin
   DestroyLabel;
   inherited Destroy;
end;

procedure TDbLblEdit.DestroyLabel;
begin
   if (FLabel <> nil) and
   not ((csDestroying in Owner.ComponentState) or
   ((FLabel.Parent <> nil) and (csDestroying in FLabel.Parent.ComponentState))) then
      begin
         if FLabel.Parent <> nil then
            FLabel.Parent.RemoveControl(FLabel);
         FLabel.Destroy;
      end;
end;

procedure TDbLblEdit.WMEnable(var Msg:TMessage);
begin
   if FLabel <> nil then
      begin
         if FLabel is TEnhLabel then
            TEnhLabel(FLabel).Disabled := not Boolean(Msg.wParam)
         else
            FLabel.Enabled := Boolean(Msg.wParam);
      end;
   inherited;
end;

procedure TDbLblEdit.SetBounds(ALeft,ATop,AWidth,AHeight:Integer);
begin
   inherited SetBounds(ALeft,ATop,AWidth,AHeight);
   if (FLabel <> nil) and not (csLoading in ComponentState) then
      DrawLabel;
end;

procedure TDbLblEdit.InitLabel;
begin
   if FLabel = nil then
      begin
         FLabel := TEnhLabel.Create(Owner);
         FLabel.Parent := Parent;
         FLabel.Name := LabelPrefix+Name;
         FLabel.FocusControl := Self;
         FLabel.Visible := False;
         TEnhLabel(FLabel).OnNewBounds := OnLabelNewBounds;
      end;
end;

procedure TDbLblEdit.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FLabel) then FLabel := nil;
end;

procedure TDbLblEdit.OnLabelNewBounds(Sender: TObject);
begin
   if FRepositionLabel and (FLabelPos <> lpCustom) and
   not ((csLoading in ComponentState) or (csLoading in FLabel.ComponentState)) then
      DrawLabel;
end;

function TDbLblEdit.HasLabel:Boolean;
begin
   Result := (FLabel <> nil) and (FLabel.Caption <> '');
end;

procedure TDbLblEdit.Loaded;
begin
   inherited Loaded;
   if HasLabel and FLabel.AutoSize then
      begin
         FLabel.AutoSize := False;
         FLabel.AutoSize := True;
      end;
   DrawLabel;
   if (FLabel <> nil) and not (csDesigning in ComponentState) then
      if FLabel is TEnhLabel then
         TEnhLabel(FLabel).Disabled := not Enabled
      else
         FLabel.Enabled := Enabled;
end;

procedure TDbLblEdit.VisibleChanging;
begin
   if FLabel <> nil then
      if Visible then
         FLabel.Visible := False
      else
         begin
            if not (csLoading in ComponentState) then DrawLabel;
            FLabel.Visible := True;
         end;
   inherited VisibleChanging;
end;

procedure TDbLblEdit.DrawLabel;
var LLeft,LTop:Integer;
begin
   if FLabel <> nil then
      begin
         case FLabelPos of
            lpTop : begin
                       LLeft := Left; LTop := Top - (FLabel.Height+FMargin);
                    end;
           lpLeft : begin
                       LLeft := Left - (FLabel.Width+FMargin);
                       LTop := Top + AdjLabelToControl;
                    end;
         lpBottom : begin
                       LLeft := Left; LTop := Top + Height + FMargin;
                    end;
          lpRight : begin
                       LLeft := Left + Width + FMargin;
                       LTop := Top + AdjLabelToControl;
                    end;
         end;
         if FLabelPos <> lpCustom then
            begin
               FRepositionLabel := False;
               FLabel.Setbounds(LLeft,LTop,FLabel.Width,FLabel.Height);
               FRepositionLabel := True;
            end;
         FLabel.Visible := Visible;
      end;
end;

procedure TDbLblEdit.SetLabelCaption(const AString:string);
begin
   if not (csLoading in ComponentState) then
      begin
         if FLabel = nil then
            begin
               FRepositionLabel := False;
               InitLabel;
            end;
         FRepositionLabel := True;
         if FAddColon and (AString <> '') and (AString[Length(AString)] <> ':') then
            FLabel.Caption := AString + ':'
         else
            FLabel.Caption := AString;
      end;
end;

function TDbLblEdit.GetLabelCaption:string;
begin
   if HasLabel then
      begin
         Result := FLabel.Caption;
         if FAddColon then Delete(Result,Length(Result),1);
      end
   else
      Result := '';
end;

procedure TDbLblEdit.SetLabelPos(APos:TLabelPos);
begin
   if csLoading in ComponentState then
      FLabelPos := APos
   else if FLabelPos <> APos then
      begin
         if (FLabel <> nil) and (FLabel.Parent <> Parent) then
            raise ELabeledControl.ShowError(ElcDiffParent,Name);
         if DefaultLabelPos = lpTop then
            begin
               if (FLabelPos = lpTop) and (APos <> lpBottom) and (FMargin = 2)
               and (csDesigning in ComponentState) then
                  FMargin := 8;
            end
         else
            begin
               if (FLabelPos = DefaultLabelPos) and (APos = lpTop) and (FMargin > 2)
               and (csDesigning in ComponentState) then
                  FMargin := 2;
            end;
         FLabelPos := APos;
         DrawLabel;
      end;
end;

procedure TDbLblEdit.SetMargin(AMargin:Integer);
begin
   if csLoading in ComponentState then
      FMargin := AMargin
   else if FMargin <> AMargin then
      begin
         FMargin := AMargin;
         DrawLabel;
      end;
end;

procedure TDbLblEdit.SetAddColon(SetOn:Boolean);
var LabelStr:string; 
begin
   if csLoading in ComponentState then
      FAddColon := SetOn
   else if FAddColon <> SetOn then
      begin
         LabelStr := GetLabelCaption;
         FAddColon := SetOn;
         if (LabelStr <> '') then
            SetLabelCaption(LabelStr);
      end;
end;

procedure TDbLblEdit.SetLabel(ALabel:TLabel);
var AString:string;
begin
   if FLabel <> ALabel then
      if (ALabel = nil) or ((ALabel.FocusControl is ClassType)
      and (ALabel.FocusControl <> Self)) then
         FLabel := nil
      else if csLoading in ComponentState then
         begin
            FLabel := ALabel;
            if FLabel is TEnhLabel then
               TEnhLabel(FLabel).OnNewBounds := OnLabelNewBounds;
         end
      else
         begin
            if ALabel.Owner <> Owner then
               raise ELabeledControl.ShowError(ElcDiffOwner,Name);
            DestroyLabel;
            FLabelPos := lpCustom;
            FLabel := ALabel;
            if FLabel.Name = '' then
               FLabel.Name := LabelPrefix+Name;
            FLabel.FocusControl := Self;
            if FLabel is TEnhLabel then
               TEnhLabel(FLabel).OnNewBounds := OnLabelNewBounds;
            AString := FLabel.Caption;
            LabelColon := (AString[Length(AString)] = ':');
         end;
end;

constructor TDbLblMemo.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   FRepositionLabel := True;
   FLabelPos := DefaultLabelPos;
   FMargin := DefaultMargin;
   FAddColon := DefaultAddColon;
end;

destructor TDbLblMemo.Destroy;
begin
   DestroyLabel;
   inherited Destroy;
end;

procedure TDbLblMemo.DestroyLabel;
begin
   if (FLabel <> nil) and
   not ((csDestroying in Owner.ComponentState) or
   ((FLabel.Parent <> nil) and (csDestroying in FLabel.Parent.ComponentState))) then
      begin
         if FLabel.Parent <> nil then
            FLabel.Parent.RemoveControl(FLabel);
         FLabel.Destroy;
      end;
end;

procedure TDbLblMemo.WMEnable(var Msg:TMessage);
begin
   if FLabel <> nil then
      begin
         if FLabel is TEnhLabel then
            TEnhLabel(FLabel).Disabled := not Boolean(Msg.wParam)
         else
            FLabel.Enabled := Boolean(Msg.wParam);
      end;
   inherited;
end;

procedure TDbLblMemo.SetBounds(ALeft,ATop,AWidth,AHeight:Integer);
begin
   inherited SetBounds(ALeft,ATop,AWidth,AHeight);
   if (FLabel <> nil) and not (csLoading in ComponentState) then
      DrawLabel;
end;

procedure TDbLblMemo.InitLabel;
begin
   if FLabel = nil then
      begin
         FLabel := TEnhLabel.Create(Owner);
         FLabel.Parent := Parent;
         FLabel.Name := LabelPrefix+Name;
         FLabel.FocusControl := Self;
         FLabel.Visible := False;
         TEnhLabel(FLabel).OnNewBounds := OnLabelNewBounds;
      end;
end;

procedure TDbLblMemo.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FLabel) then FLabel := nil;
end;

procedure TDbLblMemo.OnLabelNewBounds(Sender: TObject);
begin
   if FRepositionLabel and (FLabelPos <> lpCustom) and
   not ((csLoading in ComponentState) or (csLoading in FLabel.ComponentState)) then
      DrawLabel;
end;

function TDbLblMemo.HasLabel:Boolean;
begin
   Result := (FLabel <> nil) and (FLabel.Caption <> '');
end;

procedure TDbLblMemo.Loaded;
begin
   inherited Loaded;
   if HasLabel and FLabel.AutoSize then
      begin
         FLabel.AutoSize := False;
         FLabel.AutoSize := True;
      end;
   DrawLabel;
   if (FLabel <> nil) and not (csDesigning in ComponentState) then
      if FLabel is TEnhLabel then
         TEnhLabel(FLabel).Disabled := not Enabled
      else
         FLabel.Enabled := Enabled;
end;

procedure TDbLblMemo.VisibleChanging;
begin
   if FLabel <> nil then
      if Visible then
         FLabel.Visible := False
      else
         begin
            if not (csLoading in ComponentState) then DrawLabel;
            FLabel.Visible := True;
         end;
   inherited VisibleChanging;
end;

procedure TDbLblMemo.DrawLabel;
var LLeft,LTop:Integer;
begin
   if FLabel <> nil then
      begin
         case FLabelPos of
            lpTop : begin
                       LLeft := Left; LTop := Top - (FLabel.Height+FMargin);
                    end;
           lpLeft : begin
                       LLeft := Left - (FLabel.Width+FMargin);
                       LTop := Top + AdjLabelToControl;
                    end;
         lpBottom : begin
                       LLeft := Left; LTop := Top + Height + FMargin;
                    end;
          lpRight : begin
                       LLeft := Left + Width + FMargin;
                       LTop := Top + AdjLabelToControl;
                    end;
         end;
         if FLabelPos <> lpCustom then
            begin
               FRepositionLabel := False;
               FLabel.Setbounds(LLeft,LTop,FLabel.Width,FLabel.Height);
               FRepositionLabel := True;
            end;
         FLabel.Visible := Visible;
      end;
end;

procedure TDbLblMemo.SetLabelCaption(const AString:string);
begin
   if not (csLoading in ComponentState) then
      begin
         if FLabel = nil then
            begin
               FRepositionLabel := False;
               InitLabel;
            end;
         FRepositionLabel := True;
         if FAddColon and (AString <> '') and (AString[Length(AString)] <> ':') then
            FLabel.Caption := AString + ':'
         else
            FLabel.Caption := AString;
      end;
end;

function TDbLblMemo.GetLabelCaption:string;
begin
   if HasLabel then
      begin
         Result := FLabel.Caption;
         if FAddColon then Delete(Result,Length(Result),1);
      end
   else
      Result := '';
end;

procedure TDbLblMemo.SetLabelPos(APos:TLabelPos);
begin
   if csLoading in ComponentState then
      FLabelPos := APos
   else if FLabelPos <> APos then
      begin
         if (FLabel <> nil) and (FLabel.Parent <> Parent) then
            raise ELabeledControl.ShowError(ElcDiffParent,Name);
         if DefaultLabelPos = lpTop then
            begin
               if (FLabelPos = lpTop) and (APos <> lpBottom) and (FMargin = 2)
               and (csDesigning in ComponentState) then
                  FMargin := 8;
            end
         else
            begin
               if (FLabelPos = DefaultLabelPos) and (APos = lpTop) and (FMargin > 2)
               and (csDesigning in ComponentState) then
                  FMargin := 2;
            end;
         FLabelPos := APos;
         DrawLabel;
      end;
end;

procedure TDbLblMemo.SetMargin(AMargin:Integer);
begin
   if csLoading in ComponentState then
      FMargin := AMargin
   else if FMargin <> AMargin then
      begin
         FMargin := AMargin;
         DrawLabel;
      end;
end;

procedure TDbLblMemo.SetAddColon(SetOn:Boolean);
var LabelStr:string; 
begin
   if csLoading in ComponentState then
      FAddColon := SetOn
   else if FAddColon <> SetOn then
      begin
         LabelStr := GetLabelCaption;
         FAddColon := SetOn;
         if (LabelStr <> '') then
            SetLabelCaption(LabelStr);
      end;
end;

procedure TDbLblMemo.SetLabel(ALabel:TLabel);
var AString:string;
begin
   if FLabel <> ALabel then
      if (ALabel = nil) or ((ALabel.FocusControl is ClassType)
      and (ALabel.FocusControl <> Self)) then
         FLabel := nil
      else if csLoading in ComponentState then
         begin
            FLabel := ALabel;
            if FLabel is TEnhLabel then
               TEnhLabel(FLabel).OnNewBounds := OnLabelNewBounds;
         end
      else
         begin
            if ALabel.Owner <> Owner then
               raise ELabeledControl.ShowError(ElcDiffOwner,Name);
            DestroyLabel;
            FLabelPos := lpCustom;
            FLabel := ALabel;
            if FLabel.Name = '' then
               FLabel.Name := LabelPrefix+Name;
            FLabel.FocusControl := Self;
            if FLabel is TEnhLabel then
               TEnhLabel(FLabel).OnNewBounds := OnLabelNewBounds;
            AString := FLabel.Caption;
            LabelColon := (AString[Length(AString)] = ':');
         end;
end;

constructor TDbLblComboBox.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   FRepositionLabel := True;
   FLabelPos := DefaultLabelPos;
   FMargin := DefaultMargin;
   FAddColon := DefaultAddColon;
end;

destructor TDbLblComboBox.Destroy;
begin
   DestroyLabel;
   inherited Destroy;
end;

procedure TDbLblComboBox.DestroyLabel;
begin
   if (FLabel <> nil) and
   not ((csDestroying in Owner.ComponentState) or
   ((FLabel.Parent <> nil) and (csDestroying in FLabel.Parent.ComponentState))) then
      begin
         if FLabel.Parent <> nil then
            FLabel.Parent.RemoveControl(FLabel);
         FLabel.Destroy;
      end;
end;

procedure TDbLblComboBox.WMEnable(var Msg:TMessage);
begin
   if FLabel <> nil then
      begin
         if FLabel is TEnhLabel then
            TEnhLabel(FLabel).Disabled := not Boolean(Msg.wParam)
         else
            FLabel.Enabled := Boolean(Msg.wParam);
      end;
   inherited;
end;

procedure TDbLblComboBox.SetBounds(ALeft,ATop,AWidth,AHeight:Integer);
begin
   inherited SetBounds(ALeft,ATop,AWidth,AHeight);
   if (FLabel <> nil) and not (csLoading in ComponentState) then
      DrawLabel;
end;

procedure TDbLblComboBox.InitLabel;
begin
   if FLabel = nil then
      begin
         FLabel := TEnhLabel.Create(Owner);
         FLabel.Parent := Parent;
         FLabel.Name := LabelPrefix+Name;
         FLabel.FocusControl := Self;
         FLabel.Visible := False;
         TEnhLabel(FLabel).OnNewBounds := OnLabelNewBounds;
      end;
end;

procedure TDbLblComboBox.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (AComponent = FLabel) then FLabel := nil;
end;

procedure TDbLblComboBox.OnLabelNewBounds(Sender: TObject);
begin
   if FRepositionLabel and (FLabelPos <> lpCustom) and
   not ((csLoading in ComponentState) or (csLoading in FLabel.ComponentState)) then
      DrawLabel;
end;

function TDbLblComboBox.HasLabel:Boolean;
begin
   Result := (FLabel <> nil) and (FLabel.Caption <> '');
end;

procedure TDbLblComboBox.Loaded;
begin
   inherited Loaded;
   if HasLabel and FLabel.AutoSize then
      begin
         FLabel.AutoSize := False;
         FLabel.AutoSize := True;
      end;
   DrawLabel;
   if (FLabel <> nil) and not (csDesigning in ComponentState) then
      if FLabel is TEnhLabel then
         TEnhLabel(FLabel).Disabled := not Enabled
      else
         FLabel.Enabled := Enabled;
end;

procedure TDbLblComboBox.VisibleChanging;
begin
   if FLabel <> nil then
      if Visible then
         FLabel.Visible := False
      else
         begin
            if not (csLoading in ComponentState) then DrawLabel;
            FLabel.Visible := True;
         end;
   inherited VisibleChanging;
end;

procedure TDbLblComboBox.DrawLabel;
var LLeft,LTop:Integer;
begin
   if FLabel <> nil then
      begin
         case FLabelPos of
            lpTop : begin
                       LLeft := Left; LTop := Top - (FLabel.Height+FMargin);
                    end;
           lpLeft : begin
                       LLeft := Left - (FLabel.Width+FMargin);
                       LTop := Top + AdjLabelToControl;
                    end;
         lpBottom : begin
                       LLeft := Left; LTop := Top + Height + FMargin;
                    end;
          lpRight : begin
                       LLeft := Left + Width + FMargin;
                       LTop := Top + AdjLabelToControl;
                    end;
         end;
         if FLabelPos <> lpCustom then
            begin
               FRepositionLabel := False;
               FLabel.Setbounds(LLeft,LTop,FLabel.Width,FLabel.Height);
               FRepositionLabel := True;
            end;
         FLabel.Visible := Visible;
      end;
end;

procedure TDbLblComboBox.SetLabelCaption(const AString:string);
begin
   if not (csLoading in ComponentState) then
      begin
         if FLabel = nil then
            begin
               FRepositionLabel := False;
               InitLabel;
            end;
         FRepositionLabel := True;
         if FAddColon and (AString <> '') and (AString[Length(AString)] <> ':') then
            FLabel.Caption := AString + ':'
         else
            FLabel.Caption := AString;
      end;
end;

function TDbLblComboBox.GetLabelCaption:string;
begin
   if HasLabel then
      begin
         Result := FLabel.Caption;
         if FAddColon then Delete(Result,Length(Result),1);
      end
   else
      Result := '';
end;

procedure TDbLblComboBox.SetLabelPos(APos:TLabelPos);
begin
   if csLoading in ComponentState then
      FLabelPos := APos
   else if FLabelPos <> APos then
      begin
         if (FLabel <> nil) and (FLabel.Parent <> Parent) then
            raise ELabeledControl.ShowError(ElcDiffParent,Name);
         if DefaultLabelPos = lpTop then
            begin
               if (FLabelPos = lpTop) and (APos <> lpBottom) and (FMargin = 2)
               and (csDesigning in ComponentState) then
                  FMargin := 8;
            end
         else
            begin
               if (FLabelPos = DefaultLabelPos) and (APos = lpTop) and (FMargin > 2)
               and (csDesigning in ComponentState) then
                  FMargin := 2;
            end;
         FLabelPos := APos;
         DrawLabel;
      end;
end;

procedure TDbLblComboBox.SetMargin(AMargin:Integer);
begin
   if csLoading in ComponentState then
      FMargin := AMargin
   else if FMargin <> AMargin then
      begin
         FMargin := AMargin;
         DrawLabel;
      end;
end;

procedure TDbLblComboBox.SetAddColon(SetOn:Boolean);
var LabelStr:string;
begin
   if csLoading in ComponentState then
      FAddColon := SetOn
   else if FAddColon <> SetOn then
      begin
         LabelStr := GetLabelCaption;
         FAddColon := SetOn;
         if (LabelStr <> '') then
            SetLabelCaption(LabelStr);
      end;
end;

procedure TDbLblComboBox.SetLabel(ALabel:TLabel);
var AString:string;
begin
   if FLabel <> ALabel then
      if (ALabel = nil) or ((ALabel.FocusControl is ClassType)
      and (ALabel.FocusControl <> Self)) then
         FLabel := nil
      else if csLoading in ComponentState then
         begin
            FLabel := ALabel;
            if FLabel is TEnhLabel then
               TEnhLabel(FLabel).OnNewBounds := OnLabelNewBounds;
         end
      else
         begin
            if ALabel.Owner <> Owner then
               raise ELabeledControl.ShowError(ElcDiffOwner,Name);
            DestroyLabel;
            FLabelPos := lpCustom;
            FLabel := ALabel;
            if FLabel.Name = '' then
               FLabel.Name := LabelPrefix+Name;
            FLabel.FocusControl := Self;
            if FLabel is TEnhLabel then
               TEnhLabel(FLabel).OnNewBounds := OnLabelNewBounds;
            AString := FLabel.Caption;
            LabelColon := (AString[Length(AString)] = ':');
         end;
end;


procedure Register;
begin
   RegisterComponents('LblIC',[TDbLblEdit,TDbLblMemo,TDbLblComboBox]);
end;


end.
